/*
 * path: src/addcat.c
 * author: Pablo Yanez Trujillo <yanezp@informatik.uni-freiburg.de>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "addcat.h"

static int callback(void *NotUsed, int argc, char **argv, char **azColName)
{
	int* subid = NotUsed;

	if(!argv[0])
	{
		*subid = 0;
		return 0;
	}

	*subid = atoi(argv[0]);
	
	return 0;
}

int db_addcat(const char* path, SLL* args)
{
	SLL* list;
	int subid = -1, rc;
	char* cat_name, *sub_cat, *tmp;
	const char* questions[] = { "Category Name: ", "Subcategory of [blank for top category]: ", "", "" };
	char sql_command[1024];
	char *dbErrMsg = NULL;
	sqlite3* db;

	
	if(!path || !args) return 0;

	/* order: subid, name */
	list = userinput(questions, 2, args);

	if(!list)
		return 0;

	/* check input */

	cat_name = getSLL(list, 0, &subid);

	if(!subid)
	{
		fprintf(stderr, "Unexpected error.\n");
		cleanupSLL(list);
		return 0;
	}

	if(!strcmp(cat_name, ""))
	{
		fprintf(stderr, "The category name must not be blank.\n");
		cleanupSLL(list);
		return 0;
	}

	tmp = prepare_input(cat_name);

	if(tmp)
		cat_name = tmp;
	

	sub_cat = getSLL(list, 1, &subid);

        if(!subid)
        {
                fprintf(stderr, "Unexpected error.\n");
                cleanupSLL(list);
                return 0;
        }

	tmp = prepare_input(sub_cat);

	if(tmp)
		sub_cat = tmp;

        if(!strcmp(sub_cat, ""))
		subid = 0;

	db = db_connect(path);

	if(!db)
		return 0;

	if(subid)
	{
		subid = 0;
		sprintf(sql_command, "SELECT id FROM category WHERE name = '%s'", sub_cat);
		rc = sqlite3_exec(db, sql_command, callback, &subid, &dbErrMsg);

		if(rc != SQLITE_OK )
		{
			fprintf(stderr, "SQL error: %s\n", dbErrMsg);
			cleanupSLL(list);
			sqlite3_close(db);
			return 0;
		}

		if(!subid)
		{
			fprintf(stderr, "The category '%s' does not exist.\n", sub_cat);
			cleanupSLL(list);
			sqlite3_close(db);
			return 0;
		}
	}

	sprintf(sql_command, "INSERT INTO category(subid, name) VALUES ( %d, '%s')", subid, cat_name);

	rc = sqlite3_exec(db, sql_command, NULL, NULL, &dbErrMsg);
	
	if(rc != SQLITE_OK )
	{
		fprintf(stderr, "SQL error: %s\n", dbErrMsg);
		cleanupSLL(list);
		sqlite3_close(db);
		return 0;
	} 

	cleanupSLL(list);
	sqlite3_close(db);

	return 1;
}
