/*
 * filename: src/arg_parse.c
 * author: Pablo Yanez Trujillo <yanezp@informatik.uni-freiburg.de>
 */


#include <getopt.h>
#include <stdio.h>
#include <sstrings.h> /* already includes strings.h, stdlib.h, ctype.h */

#include <config.h>

#include "arg_parse.h"

int getargs(int argc, char* argv[], struct ARGS* sarg)
{
	int c, mask=0;
	char* strarg;

	sarg->arguments = createSLL();

	if(!sarg->arguments)
		return 0;

	while( (c = getopt(argc, argv, "carimvhsf-:")) != -1 )
	{
		switch(c)
		{
			case 'c':
				mask = 1;
				break;
			case 'a':
				mask = 1 << 1;
				break;
			case 's':
				mask = 1 << 2;
				 break;
			case 'r':
				mask = 1 << 3;
				  break;
			case 'i':
				mask = 1 << 4;
				  break;
			case 'f':
				mask = 1 << 5;
				  break;
			case 'm':
				mask = 1 << 6;
				  break;
			case 'v':
				  version();
				  cleanupSLL(sarg->arguments);
				  exit(EXIT_SUCCESS);
			case 'h':
				  help(argv[0], 0);
				  cleanupSLL(sarg->arguments);
				  exit(EXIT_SUCCESS);
			case '-':
				  if(!strcmp(optarg, "create"))
					  mask = 1;
				  else if(!strcmp(optarg, "addcat"))
					  mask = 1 << 1;
				  else if(!strcmp(optarg, "showcat"))
					  mask = 1 << 2;
				  else if(!strcmp(optarg, "rmcat"))
					  mask = 1 << 3;
				  else if(!strcmp(optarg, "addimage"))
					  mask = 1 << 4;
				  else if(!strcmp(optarg, "showimg"))
					  mask = 1 << 5;
				  else if(!strcmp(optarg, "rmimage"))
					  mask = 1 << 6;
				  else if(!strcmp(optarg, "version"))
				  {
					  version();
				  	  cleanupSLL(sarg->arguments);
				  	  exit(EXIT_SUCCESS);
				  } else if(!strcmp(optarg, "help")) {
				  	  cleanupSLL(sarg->arguments);
					  help(argv[0], 0);
				  	  exit(EXIT_SUCCESS);
				  }else if(!strcmp(optarg, "force")) {
					  mask |= 1 << 7;
				  } else {
					  fprintf (stderr, "Unknown option `--%s'.\n", optarg);
					  help(argv[0], 1);
					  return 0;
				  }
				  break;
			case '?':
			default:
				if (isprint (optopt))
					fprintf (stderr, "Unknown option `-%c'.\n", optopt);
				else
					fprintf (stderr, "Unknown option character `\\x%x'.\n", optopt);
				
				help(argv[0], 1);
				return 0;
		} /* switch(c) */

		if(!sarg->flag || (sarg->flag == ( mask & ~(1 << 7) )))
			sarg->flag = mask;
		else {
			fprintf(stderr, "ignoring option `");
			if(c == '-')
				fprintf(stderr, "--%s'\n", optarg);
			else
				fprintf(stderr, "-%c'\n", c);
		}

	} /* while ( c= getopt... ) */

	/* ignoring --force without -r and -m */

	if( !(sarg->flag & 1 << 3) && !(sarg->flag & 1 << 6) && (sarg->flag & 1 << 7) )
	{
		fprintf(stderr, "ignoring option `--force'\n");
		sarg->flag &= ~(1 << 7);
	}


	/* [arguments] are included iff sarg->flag != 0 */

	if(sarg->flag)
	{
		for(c = optind; c < argc; ++c)
		{
			strarg = str2sstr(argv[c]);

			if(!strarg)
			{
				fprintf(stderr, "Not enough memory\n");
				cleanupSLL(sarg->arguments);
				return 0;
			}

			if(!addSLL(sarg->arguments, (void*) strarg, 1))
			{
				cleanupSLL(sarg->arguments);
				return 0;
			}

			sarg->argc++;
		}
	}

	return 1;
}

void help(const char* name, int error)
{
	FILE* f = error ? stderr : stdout;

const char* help =
"usage: %s -c | -a | -r [ --force ] | -s | -i | -f | -m [ --force ] | -v | -h [arguments]\n\
-c, --create	creates a new database. File name is read\n\
		from /etc/aof-db.conf. See aof-db(1) for more\n\
		information. The option -c must be executed\n\
		by the superuser root.\n\
-a, --addcat	adds a new category to the database\n\
-r, --rmcat	removes a category (inclusive category images)\n\
-s, --showcat	shows saved categories\n\
-i, --addimage	adds a new image to a category\n\
-m, --rmimage	removes an image\n\
-f, --showimg	shows images under category `category`\n\
-v, --version	prints version of aof-db\n\
-h, --help	prints this help\n\n\
    --force     Functions only with -r and/or -m. If --force is not\n\
    		present, then the program will ask for a confirmation.\n\n\
You cannot use different options at the same time because the\n\
the arguments list of the options differs. See aof-db(1) for\n\
a detailed information about the arguments of each option.\n\
When arguments are not given, aof-db will ask for the information.\n\n\
This is free software under the GPL-2 licence\n\
Author: Pablo Yanez Trujillo <yanezp@informatik.uni-freiburg.de>\n";

	fprintf(f, help, name);
}

void version()
{
	printf("%s (aof database) version %s\n", PACKAGE, VERSION);
}
