/*
 * filename: src/conf_parse.c
 * author: Pablo Yanez Trujillo <yanezp@informatik.uni-freiburg.de>
 */

#include <stdlib.h>
#include <sstrings.h>

#include <config.h>

#include "conf_parse.h"
#include "utils.h"

static void config_error()
{
	const char* error = 
"The configuration file /etc/aof-db.conf has errors or it does not exist.\n\
Please configure the config file properly. See aof-db(1) for more\n\
information about the configuration.\n";

	fprintf(stderr, error);


}

int getconf(struct CONF* conf)
{
	int count = 1, eof = 0;
	char* line=NULL, *tmp=NULL, *old=NULL, *var=NULL, *value=NULL, *oldvar = NULL, *oldvalue = NULL;
	FILE* buffer;
	
	if(!conf) return 0;

	/* read line by line and parse the line,
	 * ignore blank lines or lines that begin with # */

	buffer = fopen("/etc/aof-db.conf", "r");

	if(!buffer)
	{
		fprintf(stderr, "Cannot open file `/etc/aof-db.conf'.");
		config_error();
		return 0;
	}

	while(1)
	{
		eof = 0;
		line = readfrom(buffer, NULL, '\n', NULL, &eof);

		if(!line)
		{
			if(feof(buffer)) break;
		}
		
		if(line)
		{
			old = line;

			line = ltrim(line);

			if( *line && ( *line != '#') )
			{
				/* split in XXX = XXX */
				var = str2sstr(old);
				
				if(!var)
				{
					fprintf(stderr, "Not enough memory\n");
					free(old);
					fclose(buffer);
					return 0;
				}

				value = str2sstr(old);

				if(!value)
                                {
                                        fprintf(stderr, "Not enough memory\n");
                                        free(old);
					free(var);
                                        fclose(buffer);
                                        return 0;
                                }

				tmp = strstr(var, "=");

				if(!tmp)
				{
					fprintf(stderr, "/etc/aof-db.conf(%d) parse error: invalid lvalue.\n", count);
					config_error();
					free(old);
					free(var);
					free(value);
					fclose(buffer);
					return 0;
				}

				*tmp = 0;
				oldvar = var;

				var = trim(var);

				if(!strcmp(var, ""))
				{
					fprintf(stderr, "/etc/aof-db.conf(%d) parse error: invalid rvalue.\n", count);
					config_error();
                                        free(old);
                                        free(oldvar);
                                        free(value);
                                        fclose(buffer);
                                        return 0;

				}

				tmp = strstr(value, "=");

				if(!tmp)
                                {
                                        fprintf(stderr, "/etc/aof-db.conf(%d) parse error: invalid lvalue.\n", count);
                                        config_error();
                                        free(old);
                                        free(oldvar);
                                        free(value);
                                        fclose(buffer);
                                        return 0;
                                }

				tmp++;

				oldvalue = value;

				value = trim(tmp);

				if(!strcmp(value, ""))
				{
					fprintf(stderr, "/etc/aof-db.conf(%d) parse error: invalid lvalue.\n", count);
					config_error();
					free(old);
                                        free(oldvar);
                                        free(oldvalue);
                                        fclose(buffer);
					return 0;
				}

				if(!strcmp(var, "DATABASE"))
				{
					tmp = str2sstr(value);

					if(!tmp)
					{
						fprintf(stderr, "Not enough memory\n");
						free(old);
                                        	free(oldvar);
                                        	free(oldvalue);
                                        	fclose(buffer);
                                        	return 0;
					}

					conf->db = tmp;
				} else if(!strcmp(var, "UID"))
				{
					conf->uid = atoi(value);

					if(!conf->uid && strcmp(value, "0"))
					{
						fprintf(stderr, "/etc/aof-db.conf(%d) parse error: invalid lvalue. Not a number\n", count);
						config_error();
						free(old);
						free(oldvar);
						free(oldvalue);
						fclose(buffer);
						return 0;
					}
				} else if(!strcmp(var, "GID"))
				{
					conf->gid = atoi(value);

                                        if(!conf->gid && strcmp(value, "0"))
                                        {
                                                fprintf(stderr, "/etc/aof-db.conf(%d) parse error: invalid lvalue. Not a number\n", count);
                                                config_error();
                                                free(old);
                                                free(oldvar);
                                                free(oldvalue);
                                                fclose(buffer);
                                                return 0;
                                        }
				} else
				{
					fprintf(stderr, "/etc/aof-db.conf(%d) undefined variable.\n", count);
					config_error();
					free(old);
					free(oldvar);
					free(oldvalue);
					fclose(buffer);
					return 0;
				}

				
				free(oldvalue); /* must be freed if is not use! */				
				free(oldvar);
			} /* end if ( *line ...  */

			free(old);

		} /* end if(line) */


		count++;
	}


	if(!conf->db || (conf->uid == -1) || (conf->gid == -1) )
	{
		fprintf(stderr, "/etc/aof-db.conf: incomplete config file\n");
		config_error();
		fclose(buffer);
		return 0;
	}

			
	fclose(buffer);

	return 1;
}
