/*
 * filename: src/rmcat.c
 * author: Pablo Yanez Trujillo <yanezp@informatik.uni-freiburg.de>
 */

#include <stdio.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <sstrings.h> /* includes stdlib.h, string.h and ctype.h */

#include "rmcat.h"

static int getid(void *NotUsed, int argc, char **argv, char **azColName)
{       
        int *id = NotUsed;
                
        *id = atoi(argv[0]);
                
        return 0;
}


static int addid(void *NotUsed, int argc, char **argv, char **azColName)
{
	SLL* list = NotUsed;
	int* id;

	id = malloc(sizeof(int));

	if(!id)
	{
		cleanupSLL(list);
		return 1;
	}

	*id = atoi(argv[0]);

	addSLL(list, id, 1);

	return 0;

	
}

int db_rmcat(const char* path, SLL* args, int force)
{
	SLL* list, *img_list, *cat_list;
	char* cat_name, *tmp, *orlist;
	char sql_command[1024];
	char *dbErrMsg = NULL;
	sqlite3* db;
	int got, id = -1, rc, *id_tmp, i, len, start, oldlen;
	char answer;

	const char* questions[] = { "Category name: ", "" };

	if(!path || !args) return 0;

	list = userinput(questions, sizeof(questions)/(2*sizeof(char*)), args);

	if(!list)
		return 0;

	/* check input */
	got = 0;

	cat_name = getSLL(list, 0, &got);

	if(!got)
	{
		fprintf(stderr, "Unexpected error\n");
                cleanupSLL(list);
                return 0;
	}

	if(!strcmp(cat_name, ""))
        {
                fprintf(stderr, "Image name must not be blank.\n");
                cleanupSLL(list);
                return 0;
        }

	tmp = prepare_input(cat_name);

	if(tmp)
		cat_name = tmp;

	/* get all categories id with subcotegory cat_name. 
	 * get all images id under those categories */

	sprintf(sql_command, "SELECT id FROM category WHERE name = '%s'", cat_name);

	db = db_connect(path);

        if(!db)
	{
		fprintf(stderr, "Cannot connect to database\n");
		cleanupSLL(list);
                return 0;
	}

	
	rc = sqlite3_exec(db, sql_command, getid, &id, &dbErrMsg);
	

	if(rc != SQLITE_OK )
	{
		fprintf(stderr, "SQL error: %s\n", dbErrMsg);
		cleanupSLL(list);
		sqlite3_close(db);
		return 0;
	}

	if(id == -1)
	{
		fprintf(stderr, "Category '%s' does not exists\n", cat_name);
		cleanupSLL(list);
		sqlite3_close(db);
		return 1;
	}

	cat_list = NULL;


	id_tmp = malloc(sizeof(int));

	if(!id_tmp)
	{
		fprintf(stderr, "Not enough memory\n");
		return 0;
	}

	*id_tmp = id;

	cat_list = createSLL();

	if(!cat_list)
	{
		fprintf(stderr, "Not enough memory\n");
		free(id_tmp);
		cleanupSLL(list);
		return 0;
	}

	addSLL(cat_list, id_tmp, 1);

	start = i = len = 0;

	len = 1;
	oldlen = 0;
	

	while(start != cat_list->addtimes)
	{
		oldlen++;
		for(i = start; i < len; ++i)
		{
			id_tmp = getSLL(cat_list, i, &got);
				
			sprintf(sql_command, "SELECT id FROM category WHERE subid = %d", *id_tmp);

			rc = sqlite3_exec(db, sql_command, addid, cat_list, &dbErrMsg);
	
			if(rc != SQLITE_OK )
        		{
                		fprintf(stderr, "SQL error: %s\n", dbErrMsg);
                		cleanupSLL(list);
				cleanupSLL(cat_list);
               			sqlite3_close(db);
                		return 0;
        		}

		}

		start = len;
		len = cat_list->addtimes;

	}

	/* ok, create list with all images */	

	img_list = createSLL();

	if(!img_list)
	{
		fprintf(stderr, "Not enough memory\n");
		cleanupSLL(list);
		cleanupSLL(cat_list);
		sqlite3_close(db);
		return 0;
	}

	for(i = 0; i < cat_list->addtimes; ++i)
	{
		id_tmp = getSLL(cat_list, i, &got);

		sprintf(sql_command, "SELECT id FROM image WHERE cat = %d", *id_tmp);

		rc = sqlite3_exec(db, sql_command, addid, img_list, &dbErrMsg);

		if(rc != SQLITE_OK )
		{
			fprintf(stderr, "SQL error: %s\n", dbErrMsg);
			cleanupSLL(list);
			cleanupSLL(img_list);
			cleanupSLL(cat_list);
			sqlite3_close(db);
			return 0;
		}
	}

	if(!force)
	{
		printf("Deleting the category '%s' means deleting %d sub categories and %d images. Proceed? [y/N] ", cat_name, 
				cat_list->addtimes, img_list->addtimes);
		answer = 0;
	
		scanf("%c", &answer);

		if( (answer != 'y') && (answer != 'Y') )
		{
			cleanupSLL(cat_list);
			cleanupSLL(img_list);
			cleanupSLL(list);
			sqlite3_close(db);
			return 1;
		}
	}


	orlist = NULL;

	for(i = 0; i < img_list->addtimes; ++i)
	{
		id_tmp = getSLL(img_list, i, &got);
		
		sprintf(sql_command, " id = %d OR", *id_tmp);

		tmp = appendstring(&orlist, sql_command);

		if(!tmp)
		{
			fprintf(stderr, "Not enough memory\n");
			free(orlist);
			cleanupSLL(cat_list);
        		cleanupSLL(img_list);
        		cleanupSLL(list);
        		sqlite3_close(db);
			return 0;
		}

		orlist = tmp;
	}

	if(img_list->addtimes)
	{
		/* category exists, but there are any images
		 * under this catgeory and subcategories */

		orlist[strlen(orlist) - 3] = 0;

		tmp = stringappend(&orlist, "DELETE from image WHERE ");

		if(!tmp)
		{
			fprintf(stderr, "Not enough memory\n");
                	free(orlist);
                	cleanupSLL(cat_list);
                	cleanupSLL(img_list);
                	cleanupSLL(list);
                	sqlite3_close(db);
                	return 0;
		}

		rc = sqlite3_exec(db, orlist, NULL, NULL, &dbErrMsg);

        	if(rc != SQLITE_OK )
        	{
        		fprintf(stderr, "SQL error: %s\n", dbErrMsg);
			free(orlist);
                	cleanupSLL(list);
                	cleanupSLL(img_list);
                	cleanupSLL(cat_list);
                	sqlite3_close(db);
                	return 0;
        	}
		free(orlist);
	}	

	orlist = NULL;

        for(i = 0; i < cat_list->addtimes; ++i)
        {
                id_tmp = getSLL(cat_list, i, &got);

                sprintf(sql_command, " id = %d OR", *id_tmp);

                tmp = appendstring(&orlist, sql_command);

                if(!tmp)
                {
                        fprintf(stderr, "Not enough memory\n");
                        free(orlist);
                        cleanupSLL(cat_list);
                        cleanupSLL(img_list);
                        cleanupSLL(list);
                        sqlite3_close(db);
                        return 0;
                }

                orlist = tmp;
        }

        orlist[strlen(orlist) - 3] = 0;

        tmp = stringappend(&orlist, "DELETE FROM category WHERE ");

        if(!tmp)
        {
                fprintf(stderr, "Not enough memory\n");
                free(orlist);
                cleanupSLL(cat_list);
                cleanupSLL(img_list);
                cleanupSLL(list);
                sqlite3_close(db);
                return 0;
        }

	rc = sqlite3_exec(db, orlist, NULL, NULL, &dbErrMsg);

        if(rc != SQLITE_OK )
        {
                fprintf(stderr, "SQL error: %s\n", dbErrMsg);
                free(orlist);
                cleanupSLL(list);
                cleanupSLL(img_list);
                cleanupSLL(cat_list);
                sqlite3_close(db);
                return 0;
        }
	free(orlist);
	

	cleanupSLL(cat_list);
	cleanupSLL(img_list);
	cleanupSLL(list);
	sqlite3_close(db);
	
	return 1;
}
