/*
 * filename: src/rmimage.c
 * author: Pablo Yanez Trujillo <yanezp@informatik.uni-freiburg.de>
 */

#include <stdlib.h>

#include "rmimage.h"

static int getid(void *NotUsed, int argc, char **argv, char **azColName)
{
	int *id = NotUsed;

	*id = atoi(argv[0]);

	return 0;
}

int db_rmimage(const char* path, SLL* args, int force)
{
	SLL *list;
	char* id_char;
	int got, id, rc;
	sqlite3* db;
	char sql_command[1024], *dbErrMsg = NULL, answer;

	const char* questions[] = { "Image id: ", "" };
	
	if(!path || !args)
		return 0;

	list = userinput(questions, sizeof(questions)/(2*sizeof(char*)), args);

        if(!list)
                return 0;

	got = 0;

	id_char = getSLL(list, 0, &got);

	if(!got)
	{
		fprintf(stderr, "Unexpected error\n");
		cleanupSLL(list);
		return 0;
	}

	if(!checknum(id_char, &id))
	{
		fprintf(stderr, "'%s' not a correct id\n", id_char);
		cleanupSLL(list);
		return 0;
	}

	if(!id)
	{
		fprintf(stderr, "The are not images with id 0\n");
		cleanupSLL(list);
		return 0;
	}

	db = db_connect(path);

	
	if(!db)
	{
		cleanupSLL(list);
		return 0;
	}

	sprintf(sql_command, "SELECT id FROM image WHERE id = %d\n", id);

	got = -1;

	rc = sqlite3_exec(db, sql_command, getid, &got, &dbErrMsg);

	if(rc != SQLITE_OK )
	{
		fprintf(stderr, "SQL error: %s\n", dbErrMsg);
		cleanupSLL(list);
		sqlite3_close(db);
		return 0;
	}

	if(got == -1)
	{
		fprintf(stderr, "The are not images with id %d\n", id);
		cleanupSLL(list);
		sqlite3_close(db);
		return 0;
	}

	if(!force)
	{
		printf("Are you sure you want to delete the image with id '%d'? [y/N] ", id);

		answer = 0;

		scanf("%c", &answer);

		if( (answer != 'y') && (answer != 'Y') )
		{
			cleanupSLL(list);
			sqlite3_close(db);
			return 1;
		}
	}

	sprintf(sql_command, "DELETE FROM image WHERE id = %d", id);

	rc = sqlite3_exec(db, sql_command, NULL, NULL, &dbErrMsg);

	if(rc != SQLITE_OK )
        {
                fprintf(stderr, "SQL error: %s\n", dbErrMsg);
                cleanupSLL(list);
                sqlite3_close(db);
                return 0;
        }

	cleanupSLL(list);
	sqlite3_close(db);

	return 1;
}
