/*
 * filename: src/showcat.c
 * author: Pablo Yanez Trujillo <yanezp@informatik.uni-freiburg.de>
 */

#include <stdio.h>
#include <sqlite3.h>
#include <sstrings.h> /* includes stdlib.h, string.h and ctype.h */

#include "showcat.h"

void printcat(sqlite3* db, const char* cat, int root);

int db_showcat(const char* path, SLL* args)
{
	SLL* list;
	int rc;
	sqlite3* db;
	const char* questions[] = { "Insert a category [blank for all categories]: ", "" };
	char* cat_name, *tmp;

	if(!path || !args) return 0;

	list = userinput(questions, 1, args);

	if(!list)
		return 0;

	cat_name = getSLL(list, 0, &rc);

	tmp = prepare_input(cat_name);

	if(tmp)
		cat_name = tmp;

	if(!rc)
	{
		fprintf(stderr, "Unexpected error\n");
		cleanupSLL(list);
		return 0;
	}

	db = db_connect(path);

	if(!db)
	{
		cleanupSLL(list);
		return 0;
	}

	printf("# category(id)\n");

	printcat(db, cat_name, 0);

	sqlite3_close(db);

	cleanupSLL(list);
	
	return 1;
}

static int idcall(void *NotUsed, int argc, char **argv, char **azColName)
{
	int* p = NotUsed;
	*p = atoi(argv[0]);
	return 0;
}

static int getidof(sqlite3* db, const char* cat)
{
	int rc, id = 0;
	char sql_command[1024];
	char *dbErrMsg = NULL;

	sprintf(sql_command, "SELECT id FROM category WHERE name = '%s'", cat);

	rc = sqlite3_exec(db, sql_command, idcall, &id, &dbErrMsg);

	if (rc != SQLITE_OK)
	{
		fprintf(stderr, "SQL error: %s\n", dbErrMsg);
		return 0;
	}

	return id;
}

static int callback(void *NotUsed, int argc, char **argv, char **azColName)
{
	char* id, *name;
	SLL* list = NotUsed;

	id = str2sstr(argv[0]);

	if(!id)
	{
		fprintf(stderr, "Not enough memory\n");
		return 1;
	}
	
	name = str2sstr(argv[1]);

	if(!name)
        {
                fprintf(stderr, "Not enough memory\n");
                return 1;
        }

	if(!addSLL(list, id, 1))
	{
		fprintf(stderr, "Not enough memory\n");
		return 1;
	}

	if(!addSLL(list, name, 1))
        {
                fprintf(stderr, "Not enough memory\n");
                return 1;
        }

	return 0;
}

void printcat(sqlite3* db, const char* cat, int root)
{
	int rc, id;
	char sql_command[1024], indent[10240];
	char *dbErrMsg = NULL;
	SLL* list = NULL;
	char* db_id, *db_cat;

	if(!strcmp(cat, ""))
		sprintf(sql_command, "SELECT id,name FROM category WHERE subid = 0 ORDER BY name");
	else {
		id = getidof(db, cat);

		if(!id)
			return;

		if(!root)
			printf("%s(%d)\n", cat, id);

		sprintf(sql_command, "SELECT id,name FROM category WHERE subid = %d ORDER BY name", id);
	}

	list = createSLL();

	if(!list)
	{
		fprintf(stderr, "Not enough memory\n");
		return;
	}

	rc = sqlite3_exec(db, sql_command, callback, list, &dbErrMsg);

	if (rc != SQLITE_OK)
	{
		fprintf(stderr, "SQL error: %s\n", dbErrMsg);
		return;
	}

	*indent = 0;

	if(list->addtimes)
	{
		for(id = 0; id < root; ++id)
			strcat(indent, " ");
	}

	for(id = 0; id < list->addtimes / 2; ++id)
	{
		db_id = getSLL(list, 2 * id, &rc);
		
		if(!rc)
		{
			fprintf(stderr, "Unexpected error\n");
			cleanupSLL(list);
			return;
		}

		db_cat = getSLL(list, 2 * id + 1, &rc);

		if(!rc)
                {
                        fprintf(stderr, "Unexpected error\n");
                        cleanupSLL(list);
                        return;
                }

		printf("%s%s(%s)\n", indent, db_cat, db_id);
		printcat(db, db_cat, root + 3);
	}

	cleanupSLL(list);
}
